module Data (..) where

import Time exposing (..)
import Color exposing (..)
import Maybe exposing (..)
import Random exposing (..)
import Level exposing (..)


----------------------------------general parameter----------------


gameWidth =
  1200


gameHeight =
  600


halfWidth =
  gameWidth / 2


halfHeight =
  gameHeight / 2


borderWidth =
  80


( fieldsizeX, fieldsizeY ) =
  ( (gameWidth), (gameHeight * 3 / 4) )


fallingVelo =
  150


ballVelo =
  430


( playerWidth, playerHeight ) =
  ( (gameWidth) / 100 * 15, 15 )

playerRadius =
  70


playerVelo =
  700


backgroundColour =
  rgb 240 250 240



-------------------------------------------------------------------------------
---------------------------------------Objects---------------------------------
-------------------------------------------------------------------------------


type BrickType
  = HardestBlack
  | HardBlue
  | HardPurple
  | Red
  | Brown
  | Blue
  | SoftYellow
  | SoftGreen
  | SoftOrange


type State
  = Play
  | NewBall
  | NewLevel
  | NewGame
  | Highscore


type ShapeType
  = Rectangle ( Float, Float )
  | Circle Float


type CollisionSite
  = Right
  | Left
  | Top
  | Bot
  | None


type alias GameState =
  { gameOver : Bool
  , state : State
  , player : Player
  , ballsList : List Ball
  , bricksList : List Brick
  , levels :
      List ( List (List Int), ( Int, Int ) )
  , levelNum : Int
  , fallingPowerups : List Powerup
  , activePowerups : List Powerup
  , seed : Seed
  }


defaultGame : GameState
defaultGame =
  { gameOver = False
  , state = NewLevel
  , player = defaultPlayer
  , ballsList = [ defaultBall ]
  , bricksList = []
  , levels = levelList
  , levelNum = 0
  , fallingPowerups = []
  , activePowerups = []
  , seed = initialSeed 5
  }


type alias Object a =
  { a
    | coord : ( Float, Float )
    , lastCoord : ( Float, Float )
    , velo : ( Float, Float )
    , shape : ShapeType
    , colour : Color
  }


type alias Ball =
  Object { godmode : Bool }


defaultBall : Ball
defaultBall =
  { coord = ( 0, (100 - halfHeight) )
  , lastCoord = ( 0, 0 )
  , velo = ( 5, 5 )
  , shape = Circle 6
  , colour = black
  , godmode = False
  }


getRandomBall : Seed -> ( Ball, Seed )
getRandomBall seed =
  let
    maxAngle =
      pi / 3

    ( angle, seed' ) =
      generate (float (-maxAngle) maxAngle) seed

    x =
      0

    y =
      ballVelo

    sinus =
      angle |> sin

    cosinus =
      angle |> cos

    newX =
      x * cosinus - y * sinus

    newY =
      x * sinus + y * cosinus
  in
    ( { defaultBall | velo = ( newX, newY ) }, seed' )


type alias Player =
  Object { lives : Int, score : Float, warp : Bool }


defaultPlayer : Player
defaultPlayer =
  { coord = ( 0, -halfHeight )
  , lastCoord = ( 0, 0 )
  , velo = ( 0, 0 )
  , shape = Rectangle ( playerWidth, playerHeight )
  , colour = black
  , lives = 5
  , score = 0
  , warp = False
  }


type alias Brick =
  Object
    { armor : Int
    , maxArmor : Int
    , points : Int
    }


defaultBrick : Brick
defaultBrick =
  { coord = ( -60, 0 )
  , lastCoord = ( 0, 0 )
  , velo = ( 0, 0 )
  , shape = Rectangle ( 0, 0 )
  , colour = green
  , armor = 3
  , maxArmor = 3
  , points = 1000
  }



-- idea: gets created when brick which has powerup vanishes
-- while falling, is visible falling
-- if under the paddle -> delete powerup
-- if gathered by the player, let activeTime count down, state indicated by not falling
-- while countdown, render it nicely in the corner to indicate active Powerups
-- delete when activTime is below zero


type alias Powerup =
  Object
    { power : Power
    , activeTime : Float
    , falling : Bool
    , name : String
    , textColour : Color
    }


defaultPowerup : Powerup
defaultPowerup =
  { coord = ( 0, 0 )
  , lastCoord = ( 0, 0 )
  , velo = ( 0, 0 )
  , shape = Rectangle ( 24, 24 )
  , colour = green
  , textColour = black
  , power = MultiBall
  , activeTime = 5.0
  , falling = True
  , name = "M"
  }


type Power
  = MultiBall
  | DoubleScore
  | RoundPlayer
  | FastBall
  | Enlarge
  | ExtraLife
  | Godmode
  | Warp
  | Shoot


type alias PowerupAlterations =
  { scoreMult : Float
  , newBalls : List Ball
  , playerShape : ShapeType
  , ballVeloMult : Float
  , sizeMult : Float
  , extraLife : Int
  , godmode : Bool
  , warp : Bool
  , shoot : Bool
  }


defaultPowerupAlterations =
  { scoreMult = 1
  , newBalls = []
  , playerShape = Rectangle ( playerWidth, playerHeight )
  , ballVeloMult = 1
  , sizeMult = 1
  , extraLife = 0
  , godmode = False
  , warp = False
  , shoot = False
  }


type alias Borders =
  { width : Float
  , height : Float
  , colour : Color
  }


defaultBorders : Borders
defaultBorders =
  { width = gameWidth
  , height = gameHeight
  , colour = backgroundColour
  }


borders =
  defaultBorders



--------------------------translate Level


translateMatrixToBricklist : List (List Int) -> ( Int, Int ) -> List Brick
translateMatrixToBricklist matrix levGrid =
  translateMatrixToBricklist' matrix 0 levGrid


translateMatrixToBricklist' : List (List Int) -> Float -> ( Int, Int ) -> List Brick
translateMatrixToBricklist' matrix rowNumber levelGrid =
  case matrix of
    x :: xs ->
      (translateRowToBrickList x rowNumber 0 levelGrid) ++ (translateMatrixToBricklist' xs (rowNumber + 1) levelGrid)

    [] ->
      []


translateRowToBrickList : List Int -> Float -> Float -> ( Int, Int ) -> List Brick
translateRowToBrickList list rowNumber colNumber levelGrid =
  case list of
    head :: rest ->
      (let
        ( gridWidth, gridHeight ) =
          ( fst levelGrid |> toFloat, snd levelGrid |> toFloat )

        ( bricksWidth, bricksHeight ) =
          ( (fieldsizeX / gridWidth), (fieldsizeY / gridHeight) )

        x =
          (colNumber * bricksWidth) + bricksWidth / 2 - halfWidth

        y =
          halfHeight - (rowNumber * bricksHeight) - bricksHeight / 2
       in
        case head of
          9 ->
            getBrick HardestBlack ( x, y ) ( bricksWidth, bricksHeight ) |> Just

          7 ->
            getBrick HardPurple ( x, y ) ( bricksWidth, bricksHeight ) |> Just

          6 ->
            getBrick HardBlue ( x, y ) ( bricksWidth, bricksHeight ) |> Just

          8 ->
            getBrick Blue ( x, y ) ( bricksWidth, bricksHeight ) |> Just

          5 ->
            getBrick Brown ( x, y ) ( bricksWidth, bricksHeight ) |> Just

          4 ->
            getBrick Red ( x, y ) ( bricksWidth, bricksHeight ) |> Just

          3 ->
            getBrick SoftOrange ( x, y ) ( bricksWidth, bricksHeight ) |> Just

          2 ->
            getBrick SoftYellow ( x, y ) ( bricksWidth, bricksHeight ) |> Just

          1 ->
            getBrick SoftGreen ( x, y ) ( bricksWidth, bricksHeight ) |> Just

          otherwise ->
            Nothing
      )
        |> \mBrick ->
            case mBrick of
              Just brick ->
                brick :: (translateRowToBrickList rest rowNumber (colNumber + 1) levelGrid)

              Nothing ->
                translateRowToBrickList rest rowNumber (colNumber + 1) levelGrid

    [] ->
      []


getBrick : BrickType -> ( Float, Float ) -> ( Float, Float ) -> Brick
getBrick brickType coord size =
  case brickType of
    HardestBlack ->
      getBrick' black 4 100 coord size

    HardPurple ->
      getBrick' lightPurple 3 50 coord size

    HardBlue ->
      getBrick' lightBlue 3 40 coord size

    Brown ->
      getBrick' brown 2 30 coord size

    Red ->
      getBrick' lightRed 2 20 coord size

    Blue ->
      getBrick' darkBlue 2 25 coord size

    SoftGreen ->
      getBrick' green 1 10 coord size

    SoftYellow ->
      getBrick' yellow 1 15 coord size

    SoftOrange ->
      getBrick' orange 1 18 coord size


getBrick' : Color -> Int -> Int -> ( Float, Float ) -> ( Float, Float ) -> Brick
getBrick' colour armor points coord size =
  { defaultBrick
    | colour = colour
    , armor = armor
    , maxArmor = armor
    , points = points
    , coord = coord
    , shape = Rectangle size
  }
